/**
* @author Dirk Nordmann
* @date   14.01.2002
*/
package util;

import java.util.ArrayList;

public class StringUtil 
{
  /**
  * Returns a new string resulting from replacing all occurrences of oldStr in 
  * the given string with newStr. 
  * If the substring oldStr does not occur in the character sequence represented 
  * by the given string, then the given string is returned. 
  * @param  s
  * @param  oldStr  the old string
  * @param  newStr  the new string
  * @return String  returns a new string resulting from replacing all 
  *                 occurrences of oldStr in the given string with newStr. 
  */
  public static String replace(String s, String oldStr, String newStr)
  {
  	int i = 0;
    while ((i = s.indexOf(oldStr,i)) >= 0)
    {
      s = s.substring(0,i) + newStr + s.substring(i + oldStr.length(),s.length());
      i += newStr.length();
    }
    return s;
  }

  /**
  * Splits this string around matches of the given regular expression.
  *
  * The array returned by this method contains each substring of this string 
  * that is terminated by another substring that matches the given expression or
  * is terminated by the end of the string. The substrings in the array are in 
  * the order in which they occur in this string. If the expression does not 
  * match any part of the input then the resulting array has just one element, 
  * namely this string. 
  *
  * The string "boo:and:foo", for example, yields the following results with 
  * these parameters: 
  *
  * Regex     Result 
  *   :     { "boo", "and", "foo" } 
  *   o     { "b", "", ":and:f" } 
  *
  * (depracted with J2SE v1.4.0 where String.split is introduced).
  *
  * @param  string    the string 
  * @param  regexp    the delimiting regular expression 
  * @return String[]  the array of strings computed by splitting this string 
  *                   around matches of the given regular expression 
  */
  public static String[] split(String string, String regexp)
  {
    ArrayList al = new ArrayList();
        
    int previousLoc = 0;
    int loc = string.indexOf( regexp, previousLoc );
        
    do {
      if (loc >= 0) {
        al.add( string.substring( previousLoc, loc ) );
        previousLoc = ( loc + regexp.length() );
        loc = string.indexOf( regexp, previousLoc );
      }
    }
    while( ( loc >= 0 ) && ( previousLoc < string.length() ) );
        
    al.add( string.substring( previousLoc ) );

    String[] sa = new String[al.size()];
    for (int i = 0; i < al.size(); i++ )
      sa[i] = (String)(al.get(i));
        
    return sa;
  }
}