###################################################################################################
# _blobfields.py
#
# Implementation of classes MyFile, MyImage, etc. (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from OFS.Image import Image, File
import string
import time
# Product Imports.
from _globals import *
import _fileutil


###################################################################################################
# createBlobField
###################################################################################################
def createBlobField(self, datatype, file=''):
  blob = None
  if datatype == DT_IMAGE:
    blob = MyImage(id='',title='',file='')
  elif datatype == DT_FILE:
    blob = MyFile(id='',title='',file='')
  if blob is not None:
    blob.aq_parent = self
    blob.mediadbfile = None
    if file:
      blob.uploadFile(file,file.filename)
  return blob

###################################################################################################
# synchronizeBlobFields
###################################################################################################
def synchronizeBlobFields(self):
  #++ print "[%s.synchronizeBlobFields]:"%(self.meta_type)

  # Remove existing DTML Blob-Methods.
  title = 'blob_html'
  ids = []
  for ob in self.objectValues(['DTML Method']):
    if ob.title == title:
      ids.append(ob.id())
  try:
    #++ print "[%s.synchronizeBlobFields]: Delete %s(DTML Method)..."%(self.meta_type,str(ids))
    self.manage_delObjects(ids)
  except:
    pass

  # Create new DTML Blob-Methods.
  for key in self.getObjAttrs().keys():
    obj_attr = self.getObjAttr(key)
    datatype = self.getObjDatatypeKey(obj_attr)
    if datatype in DT_BLOBS:
      for s_lang in self.getLanguages():
        for s_preview in ['','preview']:
          req = {}
          req['lang'] = s_lang
          req['preview'] = s_preview
          myobj = self.getObjProperty(key,req)
          if myobj is not None:
            filename = myobj.filename
            filename = string.replace(filename,'%','')
            filename = string.replace(filename,' ','')
            raw = ''
  	    raw = raw + '<dtml-call "REQUEST.set(\'lang\',\'%s\')">\n'%s_lang
  	    raw = raw + '<dtml-return "blob_html(\'%s\',REQUEST,RESPONSE)">\n'%key
            try:
              self.manage_addDTMLMethod(filename,title,raw)
            except:
              pass
            for ob in self.objectValues(['DTML Method']):
              if ob.id() == filename: 
                zmethod = ob
                #++ print "[synchronizeBlobFields]: %s"%zmethod.absolute_url()


###################################################################################################

class MyBlob:

    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.download: 
    # ---------------------------------------------------------------------------------------------
    def download(self, folder):
      filename = _fileutil.getOSPath('%s/%s'%(folder,self.filename))
      _fileutil.exportObj(self,filename)
      return filename

    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.upload:
    # ---------------------------------------------------------------------------------------------
    def upload(self, folder):
      sFilename = "%s/%s"%(folder,self.filename)
      sFilename = _fileutil.getOSPath(sFilename)
      # Backup properties (otherwise manage_upload sets it).
      bk = {}
      for key in self.__xml_attrs__:
        if hasattr(self,key):
          bk[key] = getattr(self,key)
      # Read file to ZODB.
      f = open(sFilename,'rb')
      self.uploadFile(f.read(),self.filename)
      f.close()
      # Restore properties.
      for key in self.__xml_attrs__:
        if bk.has_key(key):
          setattr(self,key,bk[key])

    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.uploadFile:
    # ---------------------------------------------------------------------------------------------
    def uploadFile(self, data, filename):
      parent = self.aq_parent
      mediadb = parent.getMediaDb()
      if data:
        self.manage_upload(data,None)
        self.filename = _fileutil.extractFileName(filename)
        if mediadb is not None:
          self.mediadbfile = mediadb.storeFile(self)
          self.data = ''
    
    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.getData: 
    # ---------------------------------------------------------------------------------------------
    def getData(self):
      parent = self.aq_parent
      mediadb = parent.getMediaDb()
      mediadbfile = getattr(self,'mediadbfile',None)
      if mediadb is not None and mediadbfile is not None:
        try:
          data = mediadb.retrieveFile(mediadbfile)
        except:
          data = ''
      else:
        data = getattr(self,'data','')
      return data
    
    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.blob_html: 
    # ---------------------------------------------------------------------------------------------
    def blob_html(self, REQUEST, RESPONSE):
      RESPONSE.setHeader('Cache-Control','public, max-age=3600')
      RESPONSE.setHeader('Content-Type',getattr(self,'content_type','Unknown'))
      RESPONSE.setHeader('Content-Disposition','inline;filename=%s'%getattr(self,'filename',''))
      return self.getData()
    
    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.src: 
    # 
    #   DEPRECATED!
    # ---------------------------------------------------------------------------------------------
    src__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
    def src(self, sender, key, REQUEST):
      filename = self.getFilename()
      qs = ''      
      if not filename in sender.objectIds(['DTML Method']):
        filename = 'blob_html'
        qs = qs_append(qs,'key',key)
        qs = qs_append(qs,'lang',REQUEST['lang'])
      if isPreviewRequest(REQUEST):
        qs = qs_append(qs,'preview','preview')
      return '%s/%s%s'%(sender.absolute_url(),filename,qs)

    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.getHref:
    # ---------------------------------------------------------------------------------------------
    getHref__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
    def getHref(self, REQUEST):
      return self.src(self.aq_parent,self.key,REQUEST)

    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.getFilename: 
    #
    #	Returns filename.
    # ---------------------------------------------------------------------------------------------
    getFilename__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
    def getFilename(self):
      filename = _fileutil.extractFileName(self.filename) 
      filename = string.replace(filename,'%','')
      filename = string.replace(filename,' ','')
      return filename
    
    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.xmlGetTagName: 
    #
    #	Returns <XML> Tag-Name.
    # ---------------------------------------------------------------------------------------------
    def xmlGetTagName(self):
      return 'data'

    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.getDataSizeStr: 
    #
    #	Display string for file-size.
    # ---------------------------------------------------------------------------------------------
    getDataSizeStr__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
    def getDataSizeStr(self):
      s = ''
      data = self.getData()
      if data is not None:
        s = _fileutil.getDataSizeStr(float(len(data)))
      return s

    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.getContentType:
    #
    #	MIME-Type.
    # ---------------------------------------------------------------------------------------------
    getContentType__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
    def getContentType(self):
      return self.content_type

###################################################################################################

class MyImage(MyBlob,Image):

    __obj_attrs__ = ['size','width','height','content_type','data','filename','mediadbfile']
    __xml_attrs__ = ['width','height','content_type']
    
    def __str__(self):
      return '{{MyImage}}'

    # ---------------------------------------------------------------------------------------------
    # 	MyImage._getCopy:
    # ---------------------------------------------------------------------------------------------
    def _getCopy(self, ob=None):
      if not ob:
        ob = self
      oClone = MyImage(id='',title='',file='')
      for attr in self.__obj_attrs__:
        if hasattr(ob,attr):
          setattr(oClone,attr,getattr(ob,attr))
      return oClone

    # ---------------------------------------------------------------------------------------------
    # 	MyImage.toXml:
    # ---------------------------------------------------------------------------------------------
    def toXml(self, sender):
      sFilepath = sender.absolute_url()[len(sender.getDocumentElement().absolute_url()):]+'/'
      sFilename = _fileutil.getOSPath(_fileutil.extractFileName(getattr(self,'filename','')))
      xml = '\n<%s'%self.xmlGetTagName()
      xml = xml + ' width="%s"'%str(getattr(self,'width',''))
      xml = xml + ' height="%s"'%str(getattr(self,'height',''))
      xml = xml + ' content_type="%s"'%str(getattr(self,'content_type',''))
      xml = xml + ' filename="%s%s"'%(sFilepath,sFilename)
      xml = xml + '>'
      xml = xml + '</%s>'%self.xmlGetTagName()
      return xml

    # ---------------------------------------------------------------------------------------------
    # 	MyImage.getWidth:
    # ---------------------------------------------------------------------------------------------
    getWidth__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
    def getWidth(self):
      return self.width

    # ---------------------------------------------------------------------------------------------
    # 	MyImage.getHeight:
    # ---------------------------------------------------------------------------------------------
    getHeight__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
    def getHeight(self):
      return self.height

###################################################################################################

class MyFile(MyBlob,File):

    __obj_attrs__ = ['size','content_type','data','filename','mediadbfile']
    __xml_attrs__ = ['content_type']

    def __str__(self):
      return '{{MyFile}}'

    # ---------------------------------------------------------------------------------------------
    # 	MyFile._getCopy:
    # ---------------------------------------------------------------------------------------------
    def _getCopy(self, ob=None):
        if not ob:
          ob = self
        oClone = MyFile(id='',title='',file='')
        for attr in self.__obj_attrs__:
          if hasattr(ob,attr):
            setattr(oClone,attr,getattr(ob,attr))
        return oClone

    # ---------------------------------------------------------------------------------------------
    # 	MyFile.toXml:
    # ---------------------------------------------------------------------------------------------
    def toXml(self, sender):
      sFilepath = sender.absolute_url()[len(sender.getDocumentElement().absolute_url()):]+'/'
      sFilename = _fileutil.getOSPath(_fileutil.extractFileName(getattr(self,'filename','')))
      xml = '\n<%s'%self.xmlGetTagName()
      xml = xml + ' content_type="%s"'%str(getattr(self,'content_type',''))
      xml = xml + ' filename="%s%s"'%(sFilepath,sFilename)
      xml = xml + '>'
      xml = xml + '</%s>'%self.xmlGetTagName()
      return xml

###################################################################################################
