###################################################################################################
# _cachemanager.py
#
# Implementation of class CacheManager (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from Globals import HTMLFile
import string 
import time
import urllib
# Product Imports.
import _globals 


dct_op = {'index':'','sitemap':'sitemap','index_print':'print'}

###################################################################################################
#
#   S T A T I C   C A C H E
#
###################################################################################################

# ------------------------------------------------------------------------------------------
# _getIdFromUrl
# ------------------------------------------------------------------------------------------
def _getIdFromUrl(REQUEST):
  id = REQUEST['URL']
  id = id[:-5]
  id = id[string.rfind(id,'/')+1:]
  if string.find(id,'_') < 0 and REQUEST.has_key('lang'):
    id = '%s_%s'%(id,REQUEST['lang'])
  return _getCacheId(id)

# ------------------------------------------------------------------------------------------
# _getCacheId
# ------------------------------------------------------------------------------------------
def _getCacheId(id):
  return 'cache_' + id + '_html'

# ------------------------------------------------------------------------------------------
# clearCachePage
# ------------------------------------------------------------------------------------------
def _clearCachePage(self, id):
  if not self.isPage():
    return _clearCachePage(self.getParentNode(),id)
  if id in self.objectIds():
    if _globals.debug: print "[%s._clearCachePage]: Removing ID=%s"%(self.meta_type,id)
    self.manage_delObjects([id],None)

# ------------------------------------------------------------------------------------------
# refresh
# ------------------------------------------------------------------------------------------
def _refreshCachePage(self, id, REQUEST):
  if not self.isPage():
    return _refreshCachePage(self.getParentNode(),id,REQUEST)
  _clearCachePage(self,id)
  if _globals.debug: print "[%s._refreshCachePage]: Generating ID=%s"%(self.meta_type,id)
  REQUEST.set('__cache__',1)
  title = '*** DO NOT DELETE OR MODIFY ***'
  raw = self.index_html(self,REQUEST)
  self.manage_addDTMLMethod(id,title,raw)
  self.setCacheEmpty(0)


###################################################################################################
#
#   R E Q U E S T   B U F F E R
#
###################################################################################################

# ------------------------------------------------------------------------------------------
# getReqBuffId:
#
# Gets buffer-id in Http-Request.
#
# @throws Exception
# ------------------------------------------------------------------------------------------
def getReqBuffId(self, key, REQUEST):
  id = '%s_%s'%(self.id,key)
  parent = self.getParentNode()
  if parent is not None:
    id = '%s_%s'%(parent.id,id)
  return id

# ------------------------------------------------------------------------------------------
# getReqBuff:
#
# Gets buffer from Http-Request.
#
# @throws Exception
# ------------------------------------------------------------------------------------------
def getReqBuff(self, REQUEST):
  buff = REQUEST.get('__buff__',None)
  if buff == None:
    buff = _globals.MyClass()
  return buff

###################################################################################################
#
# ReqBuff
#
###################################################################################################
class ReqBuff:

    # ------------------------------------------------------------------------------------------
    # fetchReqBuff:
    #
    # Fetch buffered value from Http-Request.
    #
    # @throws Exception
    # ------------------------------------------------------------------------------------------
    def fetchReqBuff(self, key, REQUEST):
      if self.isCacheActive() and string.find(REQUEST['URL'],'/manage') < 0:
        buff = getReqBuff(self,REQUEST)
        reqBuffId = getReqBuffId(self,key,REQUEST)
        try:
          value = getattr(buff,reqBuffId)
          return value
        except:
          raise '%s not found in ReqBuff!'%reqBuffId
      raise 'ReqBuff is inactive!'
    
    # ------------------------------------------------------------------------------------------
    # storeReqBuff:
    #
    # Returns value and stores it in buffer of Http-Request.
    # ------------------------------------------------------------------------------------------
    def storeReqBuff(self, key, value, REQUEST):
      try:
        if self.isCacheActive() and string.find(REQUEST['URL'],'/manage') < 0:
          buff = getReqBuff(self,REQUEST)
          reqBuffId = getReqBuffId(self,key,REQUEST)
          setattr(buff,reqBuffId,value)
          REQUEST.set('__buff__',buff)
      except:
        pass
      return value
    
    
###################################################################################################
###################################################################################################
###
###   C l a s s   C a c h e a b l e O b j e c t :
###
###################################################################################################
###################################################################################################
class CacheableObject(ReqBuff):

    # ------------------------------------------------------------------------------------------
    # CacheableObject.clearCachePage: 
    # ------------------------------------------------------------------------------------------
    def clearCachePage(self):
      for s_lang in self.getLanguages():
        for s_op in dct_op.keys():
          id = _getCacheId('%s_%s'%(s_op,s_lang))
          _clearCachePage(self,id)

    # ------------------------------------------------------------------------------------------
    # CacheableObject.clearCache: 
    # ------------------------------------------------------------------------------------------
    def clearCache(self):
      if self.isPage() and self.meta_type != 'ZMSSysFolder':
        self.clearCachePage()
        for ob in self.objectValues(self.dGlobalAttrs.keys()):
          ob.clearCache()
        self.setCacheEmpty(1)

    # ------------------------------------------------------------------------------------------
    # CacheableObject.synchronizeCachePage: 
    # ------------------------------------------------------------------------------------------
    def synchronizeCachePage(self, REQUEST):
      if self.isCacheActive():
        s_lang = REQUEST['lang']
        if self.isPageElement():
          _clearCachePage(self,_getCacheId('index_%s'%s_lang))
          _clearCachePage(self,_getCacheId('index_print_%s'%s_lang))
        elif self.isPage():
          self.setCacheUpdateFlag()
          self.synchronizeCache()

    # ------------------------------------------------------------------------------------------
    # CacheableObject.isCachedPage:
    # ------------------------------------------------------------------------------------------
    def isCachedPage(self, REQUEST):
      if not self.isPage():
        return self.getParentNode().isCachedPage(REQUEST)
      id = _getIdFromUrl(REQUEST)
      found = 0
      for s_lang in self.getLanguages():
        for s_op in dct_op.keys():
          if id == _getCacheId('%s_%s'%(s_op,s_lang)):
            found = 1
      root = self.getDocumentElement().aq_parent
      rtn = 1
      rtn = rtn and found
      rtn = rtn and (not 'attr_cacheable' in self.getObjAttrs().keys() or str(self.getObjProperty('attr_cacheable',REQUEST))=='1')
      rtn = rtn and self.isCacheActive()
      rtn = rtn and not _globals.isPreviewRequest(REQUEST)
      rtn = rtn and not _globals.isCacheRequest(REQUEST)
      rtn = rtn and REQUEST['AUTHENTICATED_USER'].has_role('Anonymous')
      try: rtn = rtn and not len(REQUEST.form.keys()) > 0
      except: pass
      return rtn

    # ------------------------------------------------------------------------------------------
    # CacheableObject.getCachedPage:
    # ------------------------------------------------------------------------------------------
    def getCachedPage(self, REQUEST):
      if not self.isPage():
        return self.getParentNode().getCachedPage(REQUEST)
      id = _getIdFromUrl(REQUEST)
      self.synchronizeCache()
      if not id in self.objectIds(['DTML Method']):
        _refreshCachePage(self,id,REQUEST)
      for ob in self.objectValues(['DTML Method']):
        if ob.id() == id:
	  return ob.raw

        
###################################################################################################
###################################################################################################
###
###   C l a s s   C a c h e M a n a g e r :
###
###################################################################################################
###################################################################################################
class CacheManager(CacheableObject): 

    # Management Permissions.
    # -----------------------
    __authorPermissions__ = ()
    __editorPermissions__ = ()
    __administratorPermissions__ = (
                'manage_changeCacheProperties',
                )
    __ac_permissions__=(
                ('ZMS Administrator', __administratorPermissions__),
                ('ZMS Author', __authorPermissions__),
                ('ZMS Editor', __editorPermissions__),
                )

    ###############################################################################################
    # Cache Active
    ###############################################################################################

    # ------------------------------------------------------------------------------------------
    # CacheManager.isCacheActive: 
    # ------------------------------------------------------------------------------------------
    def isCacheActive(self):
      return getattr(self,'__cache_active__',0)

    # ------------------------------------------------------------------------------------------
    # CacheManager.getCachePeriod: 
    # ------------------------------------------------------------------------------------------
    def getCachePeriod(self):
      return getattr(self,'__cache_period__','0')


    ###############################################################################################
    # Cache Update Flag
    ###############################################################################################

    # ------------------------------------------------------------------------------------------
    # CacheManager.synchronizeCache: 
    #	IN:   forced_update	0= not forced (update once a day), 
    #				1= forced (call from management-interface).
    # ------------------------------------------------------------------------------------------
    def synchronizeCache(self, forced_update=0):
      do_update = 1
      do_update = do_update and self.getCacheUpdateFlag() > 0
      do_update = do_update and self.getCachePeriod() == '0'
      do_update = do_update or self.getCacheUpdate()
      do_update = do_update or forced_update
      if do_update:
        if _globals.debug: print "[%s.synchronizeCache]: Update Cache..."%(self.meta_type)
        if not self.isCacheEmpty():
          self.clearCache()
        else:
          if _globals.debug: print "[%s.synchronizeCache]: Cache is empty, do nothing!"%(self.meta_type)
          pass
        setattr(self,'__cache_update__',0)
        setattr(self,'__cache_time__',time.time())


    # ------------------------------------------------------------------------------------------
    # CacheManager.isCacheEmpty:
    # ------------------------------------------------------------------------------------------
    def isCacheEmpty(self):
      return getattr(self,'__cache_empty__',0)

    # ------------------------------------------------------------------------------------------
    # CacheManager.setCacheEmpty:
    # ------------------------------------------------------------------------------------------
    def setCacheEmpty(self, empty):
      setattr(self,'__cache_empty__',empty)


    # ------------------------------------------------------------------------------------------
    # CacheManager.setCacheUpdateFlag: 
    # ------------------------------------------------------------------------------------------
    def setCacheUpdateFlag(self):
      setattr(self,'__cache_update__',self.getCacheUpdateFlag()+1)

    # ------------------------------------------------------------------------------------------
    # CacheManager.getCacheUpdateFlag: 
    # ------------------------------------------------------------------------------------------
    def getCacheUpdateFlag(self):
      return getattr(self,'__cache_update__',0)


    # ------------------------------------------------------------------------------------------
    # CacheManager.getCacheUpdateTime: 
    # ------------------------------------------------------------------------------------------
    def getCacheUpdateTime(self):
      return getattr(self,'__cache_time__',time.time())


    # ------------------------------------------------------------------------------------------
    # CacheManager.getCacheUpdate: 
    # ------------------------------------------------------------------------------------------
    def getCacheUpdate(self):
      t_now = time.time()
      t_last_update = self.getCacheUpdateTime()
      return _globals.compareDate(t_last_update,t_now) == 1


    ###############################################################################################
    # CacheManager.manage_changeCacheProperties:
    #
    # Einstellungen fr Cache ndern.
    ###############################################################################################
    def manage_changeCacheProperties(self, lang, manage_lang, REQUEST): 
      """ CacheManager.manage_changeCacheProperties """

      if REQUEST['btn'] in [self.getLangStr('BTN_CHANGE',manage_lang),self.getLangStr('BTN_CLEAR',manage_lang)]:
        cache_before = self.isCacheActive() 
        
        # Change.
        # -------
        self.__cache_active__ = REQUEST.has_key('cache')
        if REQUEST.has_key('period'): 
          self.__cache_period__ = REQUEST['period']
        
        # Clear.
        # ------
        cache_after = self.isCacheActive() 
        if (cache_before != cache_after and cache_before) or \
           (REQUEST['btn'] == self.getLangStr('BTN_CLEAR',manage_lang)):
          self.synchronizeCache(1)
            
      # Return with message.
      message = urllib.quote(self.getLangStr('MSG_CHANGED',manage_lang))
      return REQUEST.RESPONSE.redirect('manage_customizeForm?lang=%s&manage_lang=%s&manage_tabs_message=%s#_cache'%(lang,manage_lang,message))

###################################################################################################
