###################################################################################################
# _objattrs.py
#
# Implementation of class ObjAttrs (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
import copy
import string
import sys
import time
import urllib
# Product Imports.
from _globals import *
import _fileutil
import _metadictmanager
import _blobfields


# ---------------------------------------------------------------------------------------------
#  _objattrs.hasobjattr:
# ---------------------------------------------------------------------------------------------
def hasobjattr(obj, attr):
  try:
    return attr in obj.__dict__.keys()
  except:
    return hasattr(obj,attr)

# ---------------------------------------------------------------------------------------------
#  _objattrs.getobjattr:
# ---------------------------------------------------------------------------------------------
def getobjattr(self, obj, obj_attr,lang):
  attr = self.getObjAttrName(obj_attr,lang)
  if hasobjattr(obj,attr):
    return getattr(obj,attr)
  else:
    if obj_attr.has_key('default'):
      return obj_attr['default']
    datatype = self.getObjDatatypeKey(obj_attr)
    if datatype == DT_BOOLEAN:
      return 1
    elif datatype == DT_DATE:
      return None 
    elif datatype == DT_DICTIONARY:
      return {}
    elif datatype == DT_LIST:
      return []
    elif datatype == DT_INT:
      return 0
    elif datatype in DT_STRINGS:
      return ''
    elif datatype in DT_BLOBS:
      return None 
    return None

# ---------------------------------------------------------------------------------------------
#  _objattrs.setobjattr:
# ---------------------------------------------------------------------------------------------
def setobjattr(self, obj, obj_attr, value, lang):
  attr = self.getObjAttrName(obj_attr,lang)
  # Assign value.
  setattr(obj,attr,value)

# ---------------------------------------------------------------------------------------------
#  _objattrs.cloneobjattr:
# ---------------------------------------------------------------------------------------------
def cloneobjattr(self, src, dst, obj_attr, lang):
  # Fetch value.
  value = getobjattr(self,src,obj_attr,lang)
  # Clone value.
  if value is not None:
    datatype = self.getObjDatatypeKey(obj_attr)
    if datatype in DT_BLOBS:
      value = value._getCopy()
    else:
      try:
        value = copy.deepcopy(value)
      except:
        try:
          value = value.copy()
        except:
          pass
  # Assign value.
  setobjattr(self,dst,obj_attr,value,lang)


# ---------------------------------------------------------------------------------------------
#  _objattrs.getLang:
# ---------------------------------------------------------------------------------------------
def getLang(self, req):
  lang = self.getPrimaryLanguage()
  if req is not None and req.has_key('lang'): 
    lang = req['lang']
  return lang
      
# ---------------------------------------------------------------------------------------------
#  _objattrs.isLangAttr:
# ---------------------------------------------------------------------------------------------
def isLangAttr(obj_attr):
  return obj_attr['lang_attr']

# ---------------------------------------------------------------------------------------------
#  _objattrs.isEmptyAttr:
# ---------------------------------------------------------------------------------------------
def isEmptyAttr(value, datatype, obj_attr):
  return (value is None) or \
         (datatype == DT_INT and obj_attr.has_key('default') and obj_attr['default'] > 0 and value == 0) or \
  	 (datatype in DT_STRINGS and len(str(value)) == 0)


###################################################################################################
###################################################################################################
###
###   O B J E C T   A T T R I B U T E S
###
###################################################################################################
###################################################################################################
class ObjAttrs:

    # ---------------------------------------------------------------------------------------------
    #	ObjAttrs.getObjAttrs:
    #
    #	Returns object-attributes and resolves meta-dictionaries.
    # ---------------------------------------------------------------------------------------------
    def getObjAttrs(self, meta_type=None):
      if self.meta_type == 'ZMSCustom':
        return self.dObjAttrs[self.meta_id]
      else:
        meta_type = nvl(meta_type,self.meta_type)
        return self.dObjAttrs[meta_type]

    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs.getObjAttr:
    # ---------------------------------------------------------------------------------------------
    def getObjAttr(self, key, meta_type=None):
      obj_attrs = self.getObjAttrs(meta_type)
      
      if obj_attrs.has_key(key):
        obj_attr = obj_attrs[key]
      else:
        obj_attr = {}
        obj_attr['key'] = key
        obj_attr['xml'] = 1
        obj_attr['datatype'] = 'string'
        obj_attr['lang_attr'] = 0
        obj_attr['lang_inherit'] = LI_ALLOWED
        obj_attr['datatype_key'] = DT_STRING
        
      # Return Object-Attribute.
      return obj_attr


    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs.getObjOptions:
    # ---------------------------------------------------------------------------------------------
    def getObjOptions(self, obj_attr, REQUEST):
      
      opt_tpl = []
      
      if obj_attr.has_key('options'):
        opts = []
        obj_attropts = obj_attr['options']
        if type(obj_attropts) is type([]):
          for i in range(0,len(obj_attropts)/2):
            opts.append((obj_attropts[i*2],obj_attropts[i*2+1]))
        elif type(obj_attropts) is type({}):
          for k in obj_attropts.keys():
            opts.append((k,obj_attropts[k]))
        
        for opt in opts:
          lang_attr = 'OPT_'
          for skey in string.split(self.getObjKey(obj_attr),'_'):
            lang_attr = lang_attr + skey[0]
          lang_attr = lang_attr + '_' + string.replace(str(opt[1]),' ','')
          lang_attr = string.upper(lang_attr)
          opt_tpl.append((str(opt[0]),self.getLangStr(lang_attr,REQUEST['manage_lang'])))
      
      return opt_tpl

    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs.getObjKey:
    # ---------------------------------------------------------------------------------------------
    def getObjKey(self, obj_attr):
      return obj_attr['key']

    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs.getObjDatatypeKey:
    # ---------------------------------------------------------------------------------------------
    def getObjDatatypeKey(self, obj_attr):
      return obj_attr['datatype_key']

    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs.getObjDatatype:
    # ---------------------------------------------------------------------------------------------
    def getObjDatatype(self, obj_attr):
      datatype = self.getObjDatatypeKey(obj_attr)
      if datatype == DT_UNKNOWN:
        return obj_attr['datatype']
      else:
        return dtMapping[dtMapping.index(datatype)-1]

    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs.getObjAttrName:
    # ---------------------------------------------------------------------------------------------
    def getObjAttrName(self, obj_attr, lang=None):
      attr = self.getObjKey(obj_attr)
      if isLangAttr(obj_attr):
        attr = '%s_%s'%(attr,lang)
      return attr

    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs.isDisabledAttr:
    # ---------------------------------------------------------------------------------------------
    def isDisabledAttr(self, obj_attr, REQUEST):
      lang = getLang(self,REQUEST)
      return not (REQUEST is not None and REQUEST.has_key('insert') and REQUEST['insert'] == 1) and \
             not (isLangAttr(obj_attr) and obj_attr['lang_inherit'] == LI_ALLOWED) and \
             not (self.getAutocommit()) and \
             not (self.getPrimaryLanguage() == lang) and \
             not (self.getDCCoverage(REQUEST) != 'obligation')


    ###############################################################################################
    #
    #  I N T E R F A C E
    #
    ###############################################################################################

    # ---------------------------------------------------------------------------------------------
    #  getObjInput:
    # ---------------------------------------------------------------------------------------------
    def getObjInput(self, key, REQUEST):
      
      #-- DEFINITION
      obj_attr = self.getObjAttr(key)
      
      #-- DATATYPE
      datatype = self.getObjDatatypeKey(obj_attr)
      
      #-- NAME
      fmName = 'form0'
      elName = self.getObjAttrName(obj_attr,REQUEST['lang'])
      
      #-- VALUE
      value = self._getObjProperty(obj_attr,REQUEST)
      
      #-- ENABLED
      enabled = not self.isDisabledAttr(obj_attr,REQUEST)
      
      #-- TYPE
      type = 'input'
      if obj_attr.has_key('type'): 
        type = obj_attr['type']
      elif datatype == DT_BOOLEAN:
        type = 'checkbox'
          
      #-- SIZE
      size = 20
      if obj_attr.has_key('size'):
        size = obj_attr['size']
      elif datatype == DT_INT:
        size = 4
      elif datatype == DT_DATE:
        size = 8
      
      #-- INPUT
      if type == 'input': 
        if datatype == DT_DATE:
          return self.getDateInput(fmName,elName,value,enabled,REQUEST)
        else:
          return self.getTextInput(fmName,elName,size,value)
      
      #-- TEXTAREA
      elif type == 'textarea':
        css = 'form-element'
        wrap = 'on'
        if hasattr(self,'getFormat'):
          s_textformat = self.getFormat(REQUEST)
          o_textformat = self.getTextFormat(s_textformat,REQUEST)
          if not o_textformat is None and \
             not o_textformat.getTag() and \
             not o_textformat.getSubTag():
            css = 'form-fixed'
            wrap = 'off'
        return self.getTextArea(fmName,elName,50,15,value,enabled,REQUEST,css,wrap)
      
      #-- CHECKBOX
      elif type == 'checkbox':
        return self.getCheckbox(fmName,elName,value,1,enabled,REQUEST)
      
      #-- SELECT
      elif type in ['select','lookup','multiselect','multilookup']:
        options = self.getObjOptions(obj_attr,REQUEST)
        multiple = type[:5] == 'multi'
        return self.getSelect(fmName,elName,value,options,multiple,enabled,REQUEST)
      
      # Return with error-message.
      return '<font class="form-text" style="color:red">%s</font>'%key
    
    
    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs.blob_html:
    # ---------------------------------------------------------------------------------------------
    def blob_html(self, key, REQUEST, RESPONSE):
      """ ObjAttrs.blob_html """
      return self.getObjProperty(key,REQUEST).blob_html(REQUEST,RESPONSE)


    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs.hasObjProperty:
    #
    #  Checks if object has specified property.
    # ---------------------------------------------------------------------------------------------
    def hasObjProperty(self, key, REQUEST):
    
      #-- REQUEST
      lang = getLang(self,REQUEST)
      
      #-- DEFINITION
      obj_attr = self.getObjAttr(key)
      
      #-- OBJECT (Live / Work).
      ob = self.getObjVersion(REQUEST)
      
      #-- ATTR
      attr = self.getObjAttrName(obj_attr,lang)
      
      #-- Return if object has property (0=false, 1=true).
      return hasobjattr(ob,attr)


    """
    ###############################################################################################
    #
    #  G E T
    #
    ###############################################################################################
    """

    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs._getObjValue:
    # ---------------------------------------------------------------------------------------------
    def _getObjValue(self, obj_attr, datatype, REQUEST):

      #-- REQUEST    
      lang = getLang(self,REQUEST)

      #-- OBJECT (Live / Work).
      ob = self.getObjVersion(REQUEST)
        
      #-- VALUE
      set, value = 0, getobjattr(self,ob,obj_attr,lang)

      #-- None List-Fields
      if datatype != DT_LIST and type(value) is type([]):
        if len(value) > 0:
          set, value = 1, value[0]
        else:
          value = None
          
      #-- Blob-Fields
      if datatype in DT_BLOBS:
        if type(value) is type(''):
          set, value = 1, None
            
      #-- Integer-Fields.
      elif datatype in DT_INTS and type(value) is type('') and len(value) > 0:
        set, value = 1, int(value)

      #-- Date-Fields.
      elif datatype == DT_DATE:
        if type(value) is type(time.time()): 
          set, value = 1, time.localtime(value)
        elif type(value) is type(''): 
          set, value = 1, self.parseLangFmtDate(value,lang,'DATE_FMT')

      #-- SET?
      if set: 
        key = self.getObjKey(obj_attr)      
        self.setObjProperty(key,value,lang)

      # Return value.
      return value

    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs._getObjProperty:
    # ---------------------------------------------------------------------------------------------
    def _getObjProperty(self, obj_attr, REQUEST):

      #-- DATATYPE
      datatype = self.getObjDatatypeKey(obj_attr)

      #-- VALUE
      value = self._getObjValue(obj_attr,datatype,REQUEST)

      # Default to primary-language.
      req_lang = getLang(self,REQUEST)
      pri_lang = self.getPrimaryLanguage()
      if isEmptyAttr(value,datatype,obj_attr) or self.isDisabledAttr(obj_attr,REQUEST):
        if req_lang != pri_lang and isLangAttr(obj_attr) and obj_attr['lang_inherit'] in [LI_OBLIGATE,LI_ALLOWED]:
          req = {}
          req['lang'] = pri_lang
          if isPreviewRequest(REQUEST):
            req['preview'] = 'preview'
          value = self._getObjProperty(obj_attr,req)

      # Helpers for Blob-Fields
      if datatype in DT_BLOBS and value is not None:
        value = value._getCopy()
        value.aq_parent = self
        value.key = self.getObjKey(obj_attr)
      
      # Return value.
      return value

    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs.getObjProperty:
    #
    #  Retrieves value for specified property.
    # ---------------------------------------------------------------------------------------------
    def getObjProperty(self, key, REQUEST=None, default=None):

      #-- [ReqBuff]: Fetch buffered value from Http-Request.
      try:
        reqBuffId = '%s_%s'%('getObjProperty',key)
        value = self.fetchReqBuff(reqBuffId,REQUEST)
        return value
      except:
      
        #-- Get value.
        obj_attr = self.getObjAttr(key)
        value = self._getObjProperty(obj_attr,REQUEST) 
      
        #-- [ReqBuff]: Returns value and stores it in buffer of Http-Request.
        return self.storeReqBuff(reqBuffId,value,REQUEST)


    """
    ###############################################################################################
    #
    #  S E T
    #
    ###############################################################################################
    """
    
    # ---------------------------------------------------------------------------------------------
    #  setReqProperty:
    #
    #  Assigns value to specified property from Request-Object.
    # ---------------------------------------------------------------------------------------------
    def setReqProperty(self, key, REQUEST):
    
        #-- REQUEST
        lang = REQUEST['lang']
        manage_lang = REQUEST['manage_lang']
        
        #-- DEFINTION
        obj_attr = self.getObjAttr(key)
        
        #-- DATATYPE
        datatype = self.getObjDatatypeKey(obj_attr)
        
        #-- ATTR
        attr = self.getObjAttrName(obj_attr,lang)
        
        #-- VALUE
        set, value = 0, REQUEST.get(attr,None)
        
        #-- Date-Fields
        if datatype == DT_DATE:
          if value is not None and type(value) is type('') and len(value) > 0:
            value = self.parseLangFmtDate(value,manage_lang,'DATE_FMT')
          else:
            value = None
          set = 1
            
        #-- Blob-Fields
        elif datatype in DT_BLOBS:
        
          # Image-Field (Dimensions)
          if datatype == DT_IMAGE:
            img = self._getObjValue(obj_attr,datatype,REQUEST)
            if img is not None:
              if REQUEST.has_key('width_%s'%attr): 
                img.width = REQUEST['width_%s'%attr]
              if REQUEST.has_key('height_%s'%attr): 
                img.height = REQUEST['height_%s'%attr]
                
          # Delete
          if REQUEST.has_key('del_%s'%attr) and int(REQUEST['del_%s'%attr]) == 1:
            set, value = 1, None
            
          # Upload
          elif value is not None and len(getattr(value,'filename','')) > 0:
            set, value = 1, value
            
          # Reset
          elif REQUEST.has_key('insert'):
            set, value = 1, None
        
        #-- Dictionary-Fields
        elif datatype == DT_DICTIONARY:
          if value is not None:
            if type(value) is type({}):
              set, value = 1, value
        
        #-- List-Fields
        elif datatype == DT_LIST:
          if value is not None:
            if type(value) is type(''):
              value = [string.strip(value)]
            if type(value) is type([]):
              set, value = 1, value
        
        #-- String-Fields
        elif datatype in DT_STRINGS:
          if value is not None:
            set, value = 1, value
        
        #-- Integer-Fields
        elif datatype in DT_INTS:
          if value is not None:
            if type(value) == type('') and len(value) == 0:
              set, value = 1, None
            else:
              set, value = 1, int(value)
          else:
            set, value = 1, 0
        
        #-- SET?
        if set: 
          #++ print "[%s.setReqProperty] %s=%s"%(self.meta_type,key,str(value))
          self.setObjProperty(key,value,lang)        

    
    # ---------------------------------------------------------------------------------------------
    #  setObjProperty:
    #
    #  Assigns value to specified property.
    # ---------------------------------------------------------------------------------------------
    def setObjProperty(self, key, value, lang=None):
    
        #-- DEFINITION
	obj_attr = self.getObjAttr(key)
	
        #-- OBJECT (Apply changes always to Work-Version!).
        ob = self.getObjVersion({'preview':'preview'})

        #-- DATATYPE
        datatype = self.getObjDatatypeKey(obj_attr)
        
        #-- VALUE
        if type(value) is type(''):
          value = string.strip(value)
          while len(value) > 0 and value[-1] == '\n':
            value = value[:-1]
        # Retrieve value from options.
        if obj_attr.has_key('options'):
          options = obj_attr['options']
          try: 
            i = options.index(int(value))
            if i%2==1: value = options[i-1]
          except:
            try:
              i = options.index(str(value))
              if i%2==1: value = options[i-1]
            except: 
              pass
        
        #-- Blob-Fields
        if datatype in DT_BLOBS:
          file = value
          if file is not None and len(str(file)) > 0 and str(file) != str(_blobfields.createBlobField(self,datatype)):
            value = _blobfields.createBlobField(self,datatype,file)
            
        #-- Date-Fields
        elif datatype == DT_DATE:
          if value is not None and type(value) is type('') and len(value) > 0:
            value = self.parseLangFmtDate(value,lang,'DATE_FMT')
            
        #-- Integer-Fields
        elif datatype in DT_INTS:
          if value is not None and type(value) is type('') and len(value) > 0:
            value = int(value)
            
        #-- String-Fields.
        elif datatype in DT_STRINGS:
          if value is None:
            value = ''
          else:
            if type(value) is type(''):
              value = string.strip(value)

        #-- SET!
        #++ print "[%s.setObjProperty] %s=%s"%(self.meta_type,key,str(value))
        setobjattr(self,ob,obj_attr,value,lang)


    """
    ###############################################################################################
    #
    #  V E R S I O N
    #
    ###############################################################################################
    """
    
    # ---------------------------------------------------------------------------------------------
    #  __hasObjProperty__:
    #
    #  Only for internal use: checks for specified property.
    # ---------------------------------------------------------------------------------------------
    def __hasObjProperty__(self, attr):
      return hasattr(self,attr) or hasattr(self.getObjVersion({'preview':'preview'}),attr)

    
    # ---------------------------------------------------------------------------------------------
    #  __delObjProperty__:
    #
    #  Only for internal use: delete specified property.
    # ---------------------------------------------------------------------------------------------
    def __delObjProperty__(self, attr):
      try: delattr(self,attr)
      except: pass
      try: delattr(self.getObjVersion({'preview':'preview'}),attr)
      except: pass

    
    # ---------------------------------------------------------------------------------------------
    #  __initObjProperty__:
    #
    #  Only for internal use: init specified property with value.
    # ---------------------------------------------------------------------------------------------
    def __initObjProperty__(self, key, value, lang=None):
    
      #-- DEFINITION
      obj_attr = self.getObjAttr(key)
      
      #-- ATTR
      attr = self.getObjAttrName(obj_attr,lang)
      
      #-- INIT
      for preview in [None,'preview']:
        req = {}
        req['preview'] = preview
        setattr(self.getObjVersion(req),attr,value)
    

    """
    ###############################################################################################
    #
    #  C L O N E
    #
    ###############################################################################################
    """
    
    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs.cloneObjAttrs:
    #
    #  Clone object-attributes.
    # ---------------------------------------------------------------------------------------------
    def cloneObjAttrs(self, src, dst, REQUEST=None):

      #-- REQUEST      
      lang = getLang(self,REQUEST)

      #-- Clone
      for key in self.getObjAttrs().keys():
        obj_attr = self.getObjAttr(key)
        # Multi-Language Attributes.
        if isLangAttr(obj_attr):
          for s_lang in self.getLanguages():
            if lang == None or lang == s_lang:
              cloneobjattr(self,src,dst,obj_attr,s_lang)
        # Others.
        else:
          if lang == None or lang == self.getPrimaryLanguage():
            cloneobjattr(self,src,dst,obj_attr,lang)


    """
    ###############################################################################################
    #
    #  D a t a   ( I m a g e / F i l e )
    #
    ###############################################################################################
    """

    #----------------------------------------------------------------------------------------------
    #	ObjAttrs.recurse_downloadRessources:
    #
    #	Download from ZODB to file-system during Export.
    #----------------------------------------------------------------------------------------------
    def recurse_downloadRessources(self, folder='.'):
      obj_attrs = self.getObjAttrs()
      myDebugger.write("[%s.recurse_downloadRessources]: %s"%(self.meta_type,self.id))
      message = ''
      # Attributes.
      for key in obj_attrs.keys():
        obj_attr = self.getObjAttr(key)
        datatype = self.getObjDatatypeKey(obj_attr)
        if datatype in DT_BLOBS:
          for s_lang in self.getLanguages():
            try:
              req = {}
              req['lang'] = s_lang
              req['preview'] = 'preview'
              ob = self._getObjValue(obj_attr,datatype,req)
              if ob is not None: 
                filepath = self.absolute_url()[len(self.getDocumentElement().absolute_url()):]
                ob.aq_parent = self
                ob.key = key
                ob.download('%s/%s'%(folder,filepath))
            except:
              myDebugger.write("#####[%s.recurse_downloadRessources]: Exception in %s.%s_%s!#####"%(self.meta_type,self.id,key,s_lang))
              myDebugger.write('-  Type: %s'%str(sys.exc_type))
              myDebugger.write('-  Value: %s'%str(sys.exc_value))
              myDebugger.write('-  Traceback: %s'%str(sys.exc_traceback))
              pass
      # Process children.
      for child in self.getChildNodes():
        message = message + child.recurse_downloadRessources(folder)
      # Return message.
      return message


    #----------------------------------------------------------------------------------------------
    #	ObjAttrs.recurse_uploadRessources:
    #
    #	Upload from file-system to ZODB during Import.
    #----------------------------------------------------------------------------------------------
    def recurse_uploadRessources(self, folder='.'):
      obj_attrs = self.getObjAttrs()
      myDebugger.write("[%s.recurse_uploadRessources]: %s"%(self.meta_type,self.id))
      message = ''
      # Attributes.
      for key in obj_attrs.keys():
        obj_attr = self.getObjAttr(key)
        datatype = self.getObjDatatypeKey(obj_attr)
        if datatype in DT_BLOBS:
          for s_lang in self.getLanguages():
            try:
              req = {}
              req['lang'] = s_lang
              req['preview'] = 'preview'
              ob = self._getObjValue(obj_attr,datatype,req)
              if ob is not None:
                ob.aq_parent = self
                ob.key = key
                ob.upload(folder)
                self.__autoCommit__(req)
            except:
              myDebugger.write("*****[%s.recurse_uploadRessources]: Exception in %s.%s_%s!*****"%(self.meta_type,self.id,key,s_lang))
              myDebugger.write('-  Type: %s'%str(sys.exc_type))
              myDebugger.write('-  Value: %s'%str(sys.exc_value))
              myDebugger.write('-  Traceback: %s'%str(sys.exc_traceback))
              pass
      # Process children.
      for child in self.getChildNodes():
        message = message + child.recurse_uploadRessources(folder)
      # Return message.
      return message


###################################################################################################
###################################################################################################
###
###   O B J E C T   A T T R I B U T E S   M A N A G E R
###
###################################################################################################
###################################################################################################
class ObjAttrsManager:

    # ---------------------------------------------------------------------------------------------
    #	ObjAttrsManager.synchronizeObjAttrs:
    #
    #	Synchronizes object-attributes.
    # ---------------------------------------------------------------------------------------------
    def synchronizeObjAttrs(self):
      #++ print '[%s.synchronizeObjAttrs]'%(self.meta_type)
      
      self.dObjAttrs = {}
      
      ##### Init ####
      
      for meta_type in self.dGlobalAttrs.keys():
      
        ##### Special Objects ####
        if meta_type == 'ZMSCustom':
          for meta_id in self.getMetaobjIds():
            obj_attrs = self.dGlobalAttrs[meta_type]['obj_attrs'].copy()
            for attr in self.getMetaobj(meta_id)['attrs']:
              if attr['type'][:3] != 'ZMS':
                dct = {}
                dct['key'] = attr['id']
                dct['label'] = attr['name']
                dct['datatype'] = attr['type']
                dct['xml'] = 1
                obj_attrs[attr['id']] = dct
            self.dObjAttrs[meta_id] = obj_attrs
      
        ##### Standard Objects ####
        else:
          obj_attrs = self.dGlobalAttrs[meta_type]['obj_attrs'].copy()
          # Meta-Dictionaries.
          if obj_attrs.has_key('$metadict'):
            del obj_attrs['$metadict']
            for attr in self.getMetadictAttrs(meta_type):
              metadictDetails = self.getMetadictDetails(attr)
              dct = {}
              dct['key'] = attr
              dct['label'] = metadictDetails.getName()
              dct['datatype'] = metadictDetails.getDatatype()
              dct['type'] = metadictDetails.getType()
              obj_attrs[attr] = dct
          self.dObjAttrs[meta_type] = obj_attrs

      ##### Default ####
      
      for meta_type in self.dObjAttrs.keys():
        
        obj_attrs = self.dObjAttrs[meta_type]
        
        for key in obj_attrs.keys():
        
          obj_attr = obj_attrs[key]
        
          #-- KEY
          obj_attr['key'] = key
      
          #-- NATIVE
          if not obj_attr.has_key('xml'): 
            obj_attr['xml'] = 1
        
          #-- DATATYPE
          if not obj_attr.has_key('datatype'): 
            obj_attr['datatype'] = 'string'
        
          #-- LANG_ATTR
          obj_attr['lang_attr'] = obj_attr['datatype'][:4] == 'Lang'
      
          #-- LANG_INHERIT
          if not obj_attr.has_key('lang_inherit'): 
            obj_attr['lang_inherit'] = LI_ALLOWED
      
          #-- OBJ_DATATYPE  
          datatype = string.lower(obj_attr['datatype'])
          if isLangAttr(obj_attr):
            datatype = datatype[4:]
          obj_attr['datatype_key'] = datatype_key(datatype)

          obj_attrs[key] = obj_attr
        
        self.dObjAttrs[meta_type] = obj_attrs

###################################################################################################