###############################################################################################
# _xmllib.py
#
# Implementation of XML-Library (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###############################################################################################

# Imports.
from Shared.DC.xml import pyexpat
import Globals
import string
import copy
# Product Imports.
import _globals
from _objattrs import *
from _blobfields import createBlobField


"""
###############################################################################################
#
#   Q U O T E
#
###############################################################################################
"""

# ---------------------------------------------------------------------------------------------
#  xml_quote_chars:
# ---------------------------------------------------------------------------------------------
def xml_quote_chars():
  chs = [ord('&'),ord('"'),ord('<'),ord('>'),ord('?')]
  for ch in range(0,255):
    if ch < 32 or ch > 126:
      chs.append(ch)
  return chs

# ---------------------------------------------------------------------------------------------
#  xml_quote:
# ---------------------------------------------------------------------------------------------
def xml_quote(srctext):
  dsttext = srctext
  for ch in xml_quote_chars():
    hch = '00' + hex(ch)[2:]
    hch = hch[-2:]
    hch = '&#x%s;'%hch
    dsttext = string.replace(dsttext,chr(ch),hch)
  return dsttext

# ---------------------------------------------------------------------------------------------
#  inv_xml_quote:
# ---------------------------------------------------------------------------------------------
def inv_xml_quote(srctext):
  dsttext = srctext
  if len(dsttext) > 0:
    for ch in xml_quote_chars():
      hch = '00' + hex(ch)[2:]
      hch = hch[-2:]
      hch = '&#x%s;'%hch
      dsttext = string.replace(dsttext,hch,chr(ch))
  return dsttext



"""
###############################################################################################
#
#   I M P O R T
#
###############################################################################################
"""

# ---------------------------------------------------------------------------------------------
#  xmlOnCharacterData:
# ---------------------------------------------------------------------------------------------
def xmlOnCharacterData(self, sData, bInCData):
  #++ _globals.myDebugger.write("[%s.xmlOnCharacterData]: sData=%s"%(self.meta_type,sData))
  #-- TAG-STACK
  if self.dTagStack.size() > 0:
    tag = self.dTagStack.pop()
    tag['cdata'] = inv_xml_quote(tag['cdata'] + sData)
    self.dTagStack.push(tag)
  pass


# ---------------------------------------------------------------------------------------------
#  xmlOnUnknownStartTag:
# ---------------------------------------------------------------------------------------------
def xmlOnUnknownStartTag(self, sTagName, dTagAttrs):
  #++ _globals.myDebugger.write("[%s.xmlOnUnknownStartTag]: sTagName=%s"%(self.meta_type,sTagName))
  #++ _globals.myDebugger.write("[%s.xmlOnUnknownStartTag]: dTagStack[%i]=%s"%(self.meta_type,self.dTagStack.size(),self.dTagStack.toString()))
  #++ _globals.myDebugger.write("[%s.xmlOnUnknownStartTag]: dValueStack[%i]=%s"%(self.meta_type,self.dValueStack.size(),self.dValueStack.toString()))
        
  #-- TAG-STACK
  tag = {'name':sTagName,'attrs':dTagAttrs,'cdata':''}
  tag['dValueStack'] = self.dValueStack.size()
  self.dTagStack.push(tag)
  
  #-- VALUE-STACK
  if sTagName == 'lang':
    self.dValueStack.push({})
  elif sTagName == 'dictionary':
    self.dValueStack.push({})
  elif sTagName == 'list':
    self.dValueStack.push([])
    
  #-- Return
  return 1  # accept any unknown tag
        
        
# ---------------------------------------------------------------------------------------------
#  xmlOnUnknownEndTag:
# ---------------------------------------------------------------------------------------------
def xmlOnUnknownEndTag(self, sTagName):
  #++ _globals.myDebugger.write("[%s.xmlOnUnknownEndTag]: sTagName=%s"%(self.meta_type,sTagName))
  #++ _globals.myDebugger.write("[%s.xmlOnUnknownEndTag]: dTagStack[%i]=%s"%(self.meta_type,self.dTagStack.size(),self.dTagStack.toString()))
  #++ _globals.myDebugger.write("[%s.xmlOnUnknownEndTag]: dValueStack[%i]=%s"%(self.meta_type,self.dValueStack.size(),self.dValueStack.toString()))
        
  #-- TAG-STACK
  tag = self.dTagStack.pop()        
  name = tag['name']
  if name==sTagName:

      attrs = tag['attrs']
      cdata = tag['cdata']
            
      #-- ITEM (DICTIONARY|LIST) --
      #----------------------------
      if sTagName in ['list','dictionary']:
        pass
      elif sTagName in ['item']:
        if tag['dValueStack'] < self.dValueStack.size():
          item = self.dValueStack.pop()
        else:
          item = cdata
        value = self.dValueStack.pop()
        if type(value) is type({}):
          key = attrs[attrs.index('key')+1]
          value[key] = item
        if type(value) is type([]):
          value.append(item)
        self.dValueStack.push(value)
              
      #-- DATA (IMAGE|FILE) --
      #-----------------------
      elif sTagName in ['data']:
        value = {}
        for attr_key in ['width','height','content_type','filename']:
          if attr_key in attrs:
            item = attrs[attrs.index(attr_key)+1]
            value[attr_key] = inv_xml_quote(item)
        self.dValueStack.push(value)
            
      #-- LANGUAGE --
      #--------------
      elif sTagName in ['lang']:
        if self.dValueStack.size() > 1:
          item = self.dValueStack.pop()
        else:
          item = cdata
        value = self.dValueStack.pop()
        value[attrs[attrs.index('id')+1]] = item
        self.dValueStack.push(value)
        ##### Object State ####
        req = {}
        req['lang'] = attrs[attrs.index('id')+1]
        req['AUTHENTICATED_USER'] = 'xml'
        self.setObjStateNew(req)
              
      #-- OBJECT-ATTRIBUTES --
      #-----------------------
      elif sTagName in self.getObjAttrs().keys():
      
      	obj_attr = self.getObjAttr(sTagName)
      	
        #-- DATATYPE
        datatype = self.getObjDatatypeKey(obj_attr)
        
        #-- Multi-Language Attributes.
        if isLangAttr(obj_attr):
          item = self.dValueStack.pop()
          if item is not None:
            for s_lang in item.keys():
              value = item[s_lang]
              if type(value) is type(''): value = inv_xml_quote(value)
              # Data
              if datatype in _globals.DT_BLOBS:
                if type(value) is type({}) and len(value.keys()) > 0:
                  ob = createBlobField(self,datatype)
                  for attr_key in value.keys():
                    if _globals.debug: print ob,attr_key,value[attr_key]
                    setattr(ob,attr_key,value[attr_key])
                  self.__initObjProperty__(sTagName,ob,s_lang)
              # Others
              else:
                self.__initObjProperty__(sTagName,value,s_lang)
                #++ _globals.myDebugger.write("[%s.xmlOnUnknownEndTag]: %s_%s=%s"%(self.meta_type,sTagName,s_lang,str(value)))
              
        #-- Complex Attributes (Dictionary|List).
        elif self.dValueStack.size() > 0:
          value = self.dValueStack.pop()
          if type(value) is type(''): value = inv_xml_quote(value)
          # Data
          if datatype in _globals.DT_BLOBS:
            if type(value) is type({}) and len(value.keys()) > 0:
              ob = createBlobField(self,datatype)
              for attr_key in value.keys():
                if _globals.debug: print ob,attr_key,value[attr_key]
                setattr(ob,attr_key,value[attr_key])
              self.__initObjProperty__(sTagName,ob,s_lang)
          # Others
          else:
            self.__initObjProperty__(sTagName,value)
          if self.dValueStack.size() > 0:
            raise "Items on self.dValueStack=%s"%self.dValueStack.toString()
        
        #-- Simple Attributes (String, Integer, etc.)
        else:
          value = cdata
          #-- XML_QUOTE
          if type(value) is type(''): 
            value = inv_xml_quote(value)
          #-- OPTIONS
          if obj_attr.has_key('options'):
            options = obj_attr['options']
            if type(options) is type([]):
              try:
                i = options.index(int(value))
                if i%2==1: value = options[i-1]
              except:
                try: 
                  i = options.index(str(value))
                  if i%2==1: value = options[i-1]
                except: 
                  pass
          self.__initObjProperty__(sTagName,value)
          
        ##### Meta-Objects ####
        if sTagName == '__attr_dc_metaobjs__':
          self.synchronizeMetaObjs()
        ##### Meta-Dictionaries ####
        elif sTagName == '__attr_dc_metadicts__':
          self.synchronizeObjAttrs()
        
        # Clear value stack.
        self.dValueStack.clear()
      
      else:
        self.dValueStack.pop()
      
      return 1 # accept matching end tag
  else:
      return 0  # don't accept any unknown tag


"""
###############################################################################################
#
#   E X P O R T
#
###############################################################################################
"""

# ---------------------------------------------------------------------------------------------
#  toXml:
# ---------------------------------------------------------------------------------------------
def toXml(value):
  xml = ''
  
  if value is not None:
    
    # Dictionaries
    if type(value) is type({}):
      if len(value.keys()) > 0:
        xml = xml + '\n<dictionary>' 
        for key in value.keys():
          xml = xml + '\n<item key="%s">%s</item>'%(xml_quote(key),toXml(value[key]))
        xml = xml + '</dictionary>'
    
    # Lists
    elif type(value) is type([]):
      if len(value) > 0:
        xml = xml + '\n<list>'
        for item in value:
          xml = xml + '\n<item>%s</item>'%toXml(item)
        xml = xml + '</list>'
    
    # Numbers.
    elif type(value) is type(1) or \
         type(value) is type(1.0):
      s_value = str(value)
      if len(s_value) > 0:
        xml = xml + str(value)
    
    # Others.
    else:
      s_value = xml_quote(str(value))
      if len(s_value) > 0:
        xml = xml + '<![CDATA[%s]]>'%str(s_value)
  
  # Return xml.
  return xml


# ---------------------------------------------------------------------------------------------
#  getAttrToXml:
# ---------------------------------------------------------------------------------------------
def getAttrToXml(self, obj_attr, REQUEST=None):
  xml = ''
  
  #-- DATATYPE
  datatype = self.getObjDatatypeKey(obj_attr)
  
  #-- VALUE
  value = self._getObjValue(obj_attr,datatype,REQUEST)
  
  if value is not None:
    
    # Retrieve value from options.
    if obj_attr.has_key('options'):
      options = obj_attr['options']
      try:
        i = options.index(int(value))
        if i%2==0: value = options[i+1]
      except:
        try:
          i = options.index(str(value))
          if i%2==0: value = options[i+1]
        except:
          pass
    
    # Objects.
    if datatype in _globals.DT_BLOBS:
      xml = xml + value.toXml(self)
    
    # Dates.
    elif datatype == _globals.DT_DATE:
      xml = xml + toXml(self.getLangFmtDate(value,self.getPrimaryLanguage(),'DATE_FMT'))
    
    # Others.
    else:
      xml = xml + toXml(value)
    
  # Return xml.
  return xml

# ---------------------------------------------------------------------------------------------
#  getObjPropertyToXml:
# ---------------------------------------------------------------------------------------------
def getObjPropertyToXml(self, obj_attr):
  xml = ''
  # Multi-Language Attributes.
  if isLangAttr(obj_attr):
    for s_lang in self.getLanguages():
      req = {'lang':s_lang}
      s_attr_xml = getAttrToXml(self,obj_attr,req)
      if len(s_attr_xml) > 0:
        xml = xml + '\n<lang id="%s">%s</lang>'%(s_lang,s_attr_xml)
  # Simple Attributes.
  else:
    xml = xml + getAttrToXml(self,obj_attr)
  # Return xml.
  return xml

# ---------------------------------------------------------------------------------------------
#  getObjToXml:
# ---------------------------------------------------------------------------------------------
def getObjToXml(self):
  if _globals.debug: print "[%s.getObjToXml]:"%(self.meta_type)
  xml = ''
  # Start tag.
  xml = xml + '\n<%s'%self.xmlGetTagName()
  xml = xml + '\n id="%s"'%self.id
  xml = xml + '>'
  # Attributes.
  for key in self.getObjAttrs().keys():
    obj_attr = self.getObjAttr(key)
    if obj_attr['xml']:
      s_obj_prop_xml = getObjPropertyToXml(self,obj_attr)
      if len(s_obj_prop_xml) > 0:
        xml = xml + '\n<%s>%s</%s>'%(key,s_obj_prop_xml,key)
  # Process children.
  if len(self.objectValues()) > 0:
    for child in self.getChildNodes():
      xml = xml + getObjToXml(child)
  # End tag.
  xml = xml + '</%s>'%self.xmlGetTagName()
  # Return xml.
  return xml


"""
###############################################################################################
#
#   P A R S E R
#
###############################################################################################
"""
class XmlAttrBuilder:
    "class XmlAttrBuilder"

    ######## class variables ########
    iBufferSize=1028 * 32   # buffer size for XML file parsing

    ###############################################################################################
    # Builder.__init__(self):
    #
    # Constructor.
    ###############################################################################################
    def __init__(self):
        """ Builder.__init__ """
        pass


    ###############################################################################################
    # Builder.parse(self, input):
    #
    # Parse a given XML document.
    #
    # IN:  input = XML document as string
    #            = XML document as file object   
    # OUT: value or None, if nothing was parsed
    ###############################################################################################
    def parse(self, input):
        """ Builder.parse """
        
        # prepare builder
        self.dValueStack = _globals.MyStack()
        self.dTagStack   = _globals.MyStack()
        
        # create parser object
        p = pyexpat.ParserCreate()
        
        # connect parser object with handler methods
        p.StartElementHandler = self.OnStartElement
        p.EndElementHandler = self.OnEndElement
        p.CharacterDataHandler = self.OnCharacterData
        p.StartCdataSectionHandler = self.OnStartCData
        p.EndCdataSectionHandler = self.OnEndCData
        p.ProcessingInstructionHandler = self.OnProcessingInstruction
        p.CommentHandler = self.OnComment
        p.StartNamespaceDeclHandler = self.OnStartNamespaceDecl
        p.EndNamespaceDeclHandler = self.OnEndNamespaceDecl
        
        #### parsing ####
        if _globals.debug: print "#### parsing ####"
        if type(input)=='string':
          # input is a string!
          rv = p.Parse(input, 1)
        else:
          # input is a file object!
          while 1:
            if Globals.DatabaseVersion == '3':
              get_transaction().commit(1)
            
            v=input.read(self.iBufferSize)
            if v=="":
              rv = 1
              break
            
            rv = p.Parse(v, 0)
            if not rv:
              break 
              
        # raise parser exception           
        if not rv:
            raise ParseError('%s at line %s' % (pyexpat.ErrorString(p.ErrorCode), p.ErrorLineNumber))
        
        return self.dValueStack.pop()


    ###############################################################################################
    # Builder.OnStartElement(self, name, attrs):
    #
    # Handler of XML-Parser: 
    # Called at the start of a XML element (resp. on occurence of a XML start tag).
    # Usually, the occurence of a XML tag induces the instanciation of a new node object. Therefore,
    # Builder contains a mapping table ("dGlobalAttrs"), that maps XML tags to python classes. The
    # newly created node object is then made current. If no matching class is found for a XML tag,
    # the event handler "xmlOnUnknownStart()" is called on the current object.
    #
    # IN: name  = element name (=tag name)
    #     attrs = dictionary of element attributes
    ###############################################################################################
    def OnStartElement(self, sTagName, dTagAttrs):
        """ Builder.OnStartElement """
        if _globals.debug: print "OnStartElement(" + str(sTagName) + "," + str(dTagAttrs) + ") "
        
        #-- TAG-STACK
        tag = {'name':sTagName,'attrs':dTagAttrs,'cdata':''}
        tag['dValueStack'] = self.dValueStack.size()
        self.dTagStack.push(tag)
        
        #-- VALUE-STACK
        if sTagName == 'dictionary':
          self.dValueStack.push({})
        elif sTagName == 'list':
          self.dValueStack.push([])


    ###############################################################################################
    # Builder.OnEndElement(self, name):
    #
    # Handler of XML-Parser: 
    # Called at the end of a XML element (resp. on occurence of a XML end tag).
    #
    # IN: name  = element name (=tag name)
    ###############################################################################################
    def OnEndElement(self, sTagName):
        """ Builder.OnEndElement """
        if _globals.debug: print "OnEndElement(" + str(sTagName) + ")"
        
        #-- TAG-STACK
        tag = self.dTagStack.pop()        
        name = tag['name']
        attrs = tag['attrs']
        cdata = tag['cdata']
        
        if name != sTagName:
          raise ParseError("Unmatching end tag (" + sTagName + ")")
        
        #-- ITEM
        if sTagName in ['item']:
          if tag['dValueStack'] < self.dValueStack.size():
            item = self.dValueStack.pop()
          else:
            item = cdata
          value = self.dValueStack.pop()
          if type(value) is type({}):
            key = attrs[attrs.index('key')+1]
            value[key] = item
          if type(value) is type([]):
            value.append(item)
          self.dValueStack.push(value)


    ###############################################################################################
    # Builder.OnCharacterData(self, data):
    #
    # Handler of XML-Parser:
    # Called after plain character data was parsed. Forwards the character data to the current 
    # node. The class attribute "bInCData" determines, wether the character data is nested in a 
    # CDATA block.
    #
    # IN: data = character data string
    ###############################################################################################
    def OnCharacterData(self, sData):
        """ Builder.OnCharacterData """
        if _globals.debug: print "OnCharacterData(" + str(sData) + ")"
        
        #-- TAG-STACK
        if self.dTagStack.size() > 0:
          tag = self.dTagStack.pop()
          if getattr(self,'bInCData',_globals.false):
            tag['cdata'] = inv_xml_quote(tag['cdata'] + sData)
          else:
            tag['cdata'] = tag['cdata'] + sData
            try:
              tag['cdata'] = int(tag['cdata'])
            except:
              pass
          self.dTagStack.push(tag)


    ###############################################################################################
    # Builder.OnStartCData(self):
    #
    # Handler of XML-Parser:
    # Called at the start of a CDATA block (resp. on occurence of the "CDATA[" tag).
    ###############################################################################################
    def OnStartCData(self):
        """ Builder.OnStartCData """
        self.bInCData=_globals.true


    ###############################################################################################
    # Builder.OnEndCData(self):
    #
    # Handler of XML-Parser:
    # Called at the end of a CDATA block (resp. on occurence of the "]" tag).
    ###############################################################################################
    def OnEndCData(self):
        """ Builder.OnEndCData """
        self.bInCData=_globals.false


    ###############################################################################################
    # Builder.OnProcessingInstruction(self, target, data):
    #
    # Handler of XML-Parser:
    # Called on occurence of a processing instruction.
    #
    # IN: target = target (processing instruction)
    #     data   = dictionary of data
    ###############################################################################################
    def OnProcessingInstruction(self, target, data):
        """ Builder.OnProcessingInstruction """
        pass  # ignored


    ###############################################################################################
    # Builder.OnComment(self, data):
    #
    # Handler of XML-Parser:
    # Called on occurence of a comment.
    #
    # IN: data = comment string
    ###############################################################################################
    def OnComment(self, data):
        """ Builder.OnComment """
        pass  # ignored


    ###############################################################################################
    # Builder.OnStartNamespaceDecl(self, prefix, uri):
    #
    # Handler of XML-Parser:
    # Called at the start of a namespace declaration.
    #
    # IN: prefix = prefix of namespace
    #     uri    = namespace identifier
    ###############################################################################################
    def OnStartNamespaceDecl(self, prefix, uri):
        """ Builder.OnStartNamespaceDecl """
        pass  # ignored


    ###############################################################################################
    # Builder.OnEndNamespaceDecl(self, prefix):
    #
    # Handler of XML-Parser:
    # Called at the end of a namespace declaration.
    #
    # IN: prefix = prefix of namespace
    ###############################################################################################
    def OnEndNamespaceDecl(self, prefix):
        """ Builder.OnEndNamespaceDecl """
        pass  # ignored

###################################################################################################