###################################################################################################
# zmslinkcontainer.py
#
# Implementation of class ZMSLinkContainer (see below).
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from Globals import HTML, HTMLFile
from OFS.SimpleItem import Item
import string 
import urllib
import sys
# Product Imports.
from zmsobject import ZMSObject
from _linklib import *
import _globals


# ------------------------------------------------------------------------------------------
# recurse_updateVersion:
#
# Update version.
# ------------------------------------------------------------------------------------------
def recurse_updateVersion(self, lang=None, manage_lang=None, REQUEST=None):
      message = ''
      
      # Process links.
      
      # ---------------------------------------------------------------------
      # 11.10.2001 - 0.8 to 0.9: 
      # Links enhanced by printmedia-Attributes.
      # ---------------------------------------------------------------------
      if hasattr(self,'attr_links'):
        old_links = getattr(self,'attr_links')
        for s_lang in self.getLanguages():
          new_links = {}
          for url in old_links.keys():
            title = old_links[url]['title_%s'%s_lang]
            if len(title) == 0:
              title = url
            new_links[url] = {}
            new_links[url]['title'] = old_links[url]['title_%s'%s_lang]
            new_links[url]['authors'] = ''
            new_links[url]['source'] = ''
            new_links[url]['annual'] = ''
            new_links[url]['vol'] = ''
            new_links[url]['p'] = ''
          setattr(self,'attr_links_%s'%s_lang,new_links)
        delattr(self,'attr_links')
      
      # ---------------------------------------------------------------------
      # 04.12.2001 - 0.8 to 0.9: 
      # printmedia-Attributes replaced by description
      # ---------------------------------------------------------------------
      for s_lang in self.getLanguages():
        if hasattr(self,'attr_links_%s'%s_lang):
          old_links = getattr(self,'attr_links_%s'%s_lang)
          if type(old_links) is type({}):
            new_links = {}
            for url in old_links.keys():
              new_links[url] = {}
              new_links[url]['title'] = old_links[url]['title']
              new_links[url]['description'] = ''
              for key in ['description','authors','source','annual','vol','p']:
                if old_links[url].has_key(key):
                  if len(old_links[url][key]) > 0:
                    if len(new_links[url]['description']) > 0:
                      new_links[url]['description'] = new_links[url]['description'] + '\n'
                    new_links[url]['description'] = new_links[url]['description'] + old_links[url][key]
            setattr(self,'attr_links_%s'%s_lang,new_links)
      
      # ---------------------------------------------------------------------
      # 05.12.2001 - 0.8 to 0.9: 
      # Allow empty urls
      # ---------------------------------------------------------------------
      for s_lang in self.getLanguages():
        if hasattr(self,'attr_links_%s'%s_lang):
          old_links = getattr(self,'attr_links_%s'%s_lang)
          if type(old_links) is type({}):
            new_links = []
            for url in old_links.keys():
              new_link = {}
              new_link['url'] = url
              new_link['title'] = old_links[url]['title']
              new_link['description'] = old_links[url]['description']
              new_links.append(new_link)
            setattr(self,'attr_links_%s'%s_lang,new_links)
      
      # ---------------------------------------------------------------------
      # 05.02.2002 - 0.8 to 0.9: 
      # ZMSLinkContainer
      # ---------------------------------------------------------------------
      if self.meta_type != 'ZMSLinkContainer':
        count = 0
        for s_lang in self.getLanguages():
          count = count + len(getattr(self,'attr_links_%s'%s_lang,[]))
        if count > 0:
          
          ##### Create ZMSLinkContainer ####
          obj = ZMSLinkContainer(self.getNewId(),self.getSortId(REQUEST)+1)
          parent = self.getParentNode()
          parent._setObject(obj.id, obj)
          
          for oItem in parent.objectValues():
            if oItem.id == obj.id:
              ##### Object State ####
              oItem.setObjStateNew(REQUEST)
              ##### Init Properties ####
              for s_lang in self.getLanguages():
                oItem.setObjProperty('attr_links',getattr(self,'attr_links_%s'%s_lang,[]),s_lang)
                oItem.setObjProperty('active',1,s_lang)
              ##### VersionManager ####
              oItem.__autoCommit__(REQUEST)
          
          ##### Normalize Sort-IDs ####
          parent.normalizeSortIds()
          
          for s_lang in self.getLanguages():
            try:
              if hasattr(self,'attr_links_%s'%s_lang):
                delattr(self,'attr_links_%s'%s_lang)
            except:
              pass
      
      # Return with message.
      return message


###################################################################################################
###################################################################################################
###   
###   C o n s t r u c t o r ( s )
###   
###################################################################################################
###################################################################################################

def manage_addZMSLinkContainer(self, lang, manage_lang, _sort_id, REQUEST):
  """ manage_addZMSLinkContainer """
  
  ##### Create ####
  obj = ZMSLinkContainer(self.getNewId(),_sort_id+1)
  self._setObject(obj.id, obj)
  
  oItem = getattr(self,obj.id)
  ##### Object State ####
  oItem.setObjStateNew(REQUEST)
  ##### Init Properties ####
  for s_lang in self.getLanguages():
    oItem.setObjProperty('active',1,s_lang)
  ##### VersionManager ####
  oItem.__autoCommit__(REQUEST)
  
  ##### Normalize Sort-IDs ####
  self.normalizeSortIds()
            
  # Return with message.        
  if REQUEST.RESPONSE:
    message = self.getLangStr('MSG_INSERTED',manage_lang)%obj.display_type(REQUEST)
    REQUEST.RESPONSE.redirect('%s/%s/manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(self.absolute_url(),obj.id,lang,manage_lang,urllib.quote(message)))


###################################################################################################
###################################################################################################
###   
###   C l a s s
###   
###################################################################################################
###################################################################################################

class ZMSLinkContainer(ZMSObject, Item): 

    # Properties.
    # -----------
    meta_type = "ZMSLinkContainer"
    icon = "misc_/zms/zmslinkcontainer.gif"
    icon_disabled = "misc_/zms/zmslinkcontainer_disabled.gif"

    # Management Options.
    # -------------------
    manage_options = ( 
	{'label': 'TAB_EDIT',    'action': 'manage_main'},
	{'label': 'TAB_HISTORY', 'action': 'manage_UndoForm'},
	{'label': 'TAB_PREVIEW', 'action': 'preview_html'}, # empty string defaults to index_html
	)

    # Management Permissions.
    # -----------------------
    __authorPermissions__ = (
		'manage','manage_main',
		'manage_changeProperties',
		'manage_moveObjUp','manage_moveObjDown',
		'manage_rollbackVersionChanges','manage_requestVersionChanges',
		)
    __editorPermissions__ = (
		'manage_commitVersionChanges',
		)
    __administratorPermissions__ = (
                )
    __ac_permissions__=(
		('ZMS Author', __authorPermissions__),
		('ZMS Editor', __editorPermissions__),
		('ZMS Administrator', __administratorPermissions__),
		)

    # Properties.
    # -----------
    __obj_attrs__ = {
        # Changed by
        'change_uid':{'datatype':'LangString','xml':0,'lang_inherit':_globals.LI_FORBIDDEN},
        'change_dt':{'datatype':'LangDate','xml':0,'lang_inherit':_globals.LI_FORBIDDEN},
        # Properties
        'sort_id':{'datatype':'string','xml':0},
        'active':{'datatype':'LangInt','type':'checkbox','lang_inherit':_globals.LI_OBLIGATE},
        'attr_links':{'datatype':'LangList'},
    }


    # Management Interface.
    manage_main = HTMLFile('dtml/zmslinkcontainer/manage_main', globals())


    """
    ###############################################################################################    
    #
    #   CONSTRUCTOR
    #
    ###############################################################################################    
    """
    
    ###############################################################################################
    # ZMSLinkContainer.recurse_updateVersion:
    #
    # Update version.
    ###############################################################################################
    def recurse_updateVersion(self, lang, manage_lang, REQUEST):
      message = ''

      # Process object.
      # [...]

      # ---------------------------------------------------------------------
      # 29.08.2002 Build #099
      # Apply information about user-id and date of change..
      # ---------------------------------------------------------------------
      if getattr(self.getDocumentElement(),'build','') < '099':
        if not self.getObjProperty('change_uid',REQUEST):
          self.setObjProperty('change_uid','zms',REQUEST['lang'])
        if not self.getObjProperty('change_dt',REQUEST):
          self.setObjProperty('change_dt',time.localtime(time.time()),REQUEST['lang'])

      # Process super-objects.
      message = message + ZMSObject.recurse_updateVersion(self,lang,manage_lang,REQUEST)
        
      # Return with message.            
      return message


    """
    ###############################################################################################    
    #
    #   Properties
    #
    ###############################################################################################    
    """
    
    ###############################################################################################
    #  ZMSLinkContainer.manage_changeProperties: 
    #
    #  Change LinkContainer properties.
    ###############################################################################################
    def manage_changeProperties(self, lang, manage_lang, REQUEST, RESPONSE): 
        """ ZMSLinkContainer.manage_changeProperties """
        
        message = ''
        if REQUEST.get('btn','') != self.getLangStr('BTN_CANCEL',manage_lang):
        
          ##### Object State #####
          self.setObjStateModified(REQUEST)
          
          ##### Delete #####
          if REQUEST['btn'] == 'delete':
            self.__del_links__([REQUEST['id']],REQUEST)
          
          ##### Move Up #####
          elif REQUEST['btn'] == 'move_up':
            self.__move_link__(int(REQUEST['id']),-1,REQUEST)
          
          ##### Move Down #####
          elif REQUEST['btn'] == 'move_down':
            self.__move_link__(int(REQUEST['id']),+1,REQUEST)
          
          ##### Change #####
          elif REQUEST['btn'] == self.getLangStr('BTN_CHANGE',manage_lang):
            self.setReqProperty('active',REQUEST)
            old_links = self.__get_links__(REQUEST)
            count = len(old_links)
            self.__init_links__(REQUEST)
            for id in range(0,count):
              url = REQUEST['url%i'%id]
              title = REQUEST['title%i'%id]
              description = REQUEST['description%i'%id]
              self.__set_link__(url,title,description,REQUEST)
          
          ##### Add #####
          elif REQUEST['btn'] == self.getLangStr('BTN_INSERT',manage_lang):
            self.setReqProperty('active',REQUEST)
            url = REQUEST['_url']
            title = REQUEST['_title']
            description = REQUEST['_description']
            self.__set_link__(url,title,description,REQUEST)
                
          ##### VersionManager ####
          self.__autoCommit__(REQUEST)
          
          # Return with message.
          message = self.getLangStr("MSG_CHANGED",manage_lang)
          return RESPONSE.redirect('manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(lang,manage_lang,urllib.quote(message)))
        
        else:
          # Return to parent.
          self.checkIn(REQUEST)
          return RESPONSE.redirect('%s/manage_main?lang=%s&manage_lang=%s#_%s'%(self.getParentNode().absolute_url(),lang,manage_lang,self.id))


    # -------------------------------------------------------------------------------------------------
    #	ZMSLinkContainer.isPageElement
    # ---------------------------------------------------------------------------------------------
    def isPageElement(self): return 1

    # -------------------------------------------------------------------------------------------------
    #	ZMSLinkContainer.getTitlealt
    # ---------------------------------------------------------------------------------------------
    def getTitlealt(self,REQUEST): return self.display_type(REQUEST)

    # ------------------------------------------------------------------------------------------
    # ZMSLinkContainer.__init_links__:
    # 
    # Initializes list of links.
    # ------------------------------------------------------------------------------------------
    def __init_links__(self, REQUEST):
      self.setObjProperty('attr_links',[],REQUEST['lang'])

    # ------------------------------------------------------------------------------------------
    # 	ZMSLinkContainer.__get_links__:
    #	
    #	Returns list of links.
    # ------------------------------------------------------------------------------------------
    def __get_links__(self, REQUEST):
      if not self.hasObjProperty('attr_links',REQUEST):
        self.__init_links__(REQUEST)
      return self.getObjProperty('attr_links',REQUEST)

    # ------------------------------------------------------------------------------------------
    # 	ZMSLinkContainer.getLinkDetails:
    #	
    #	Returns list of link-details.
    # ------------------------------------------------------------------------------------------
    def getLinkDetails(self, REQUEST):
    
      # Repair internal links.
      obs = []
      c = 0
      for ob in self.__get_links__(REQUEST):
        oLinkDetail = LinkDetails(ob)
        url = oLinkDetail.getUrl()
        if isInternalLink(url):
          dst = oLinkDetail.getLinkObj(self,REQUEST)
          if dst is not None:
            internal_url = getInternalLink(dst.relative_obj_path()) 
            if url != internal_url:
              #++ print "[%s.getLinkDetails]: Repair %s!"%(self.meta_type,internal_url)
              ob['url'] = internal_url
              c = c + 1
        obs.append(ob)
      # Repair attribute (if necessary).
      if c > 0: self.setObjProperty('attr_links',obs,REQUEST['lang'])
      
      rtn = []
      for ob in self.__get_links__(REQUEST):
        oLinkDetail = LinkDetails(ob)
        rtn.append(oLinkDetail)
      
      return rtn

    # ------------------------------------------------------------------------------------------
    #	ZMSLinkContainer.getLinkList: 
    #
    #	Returns list of URLs of links.
    # ------------------------------------------------------------------------------------------
    def getLinkList(self, REQUEST, allow_none=0):
      rtn = []
      for oLinkDetail in self.getLinkDetails(REQUEST):
        url = oLinkDetail.getUrl()
        dct = {}
        dct['internal'] = isInternalLink(url)
	dct['src'] = self
	dct['dst'] = oLinkDetail.getLinkObj(self,REQUEST)
        dct['url'] = oLinkDetail.getDisplayUrl(self,REQUEST)
        dct['title'] = oLinkDetail.getDisplayTitle(self,REQUEST)
	dct['description'] = oLinkDetail.getDisplayDescription(self,REQUEST)
        if dct['url'] is not None or allow_none:
          rtn.extend([dct])
      return rtn

    # ------------------------------------------------------------------------------------------
    #	ZMSLinkContainer.__set_link__: 
    #
    #	Set/add link with specified values.
    # ------------------------------------------------------------------------------------------
    def __set_link__(self, url, title, description, REQUEST):
        attr_links = self.__get_links__(REQUEST)
        # Append link.
        new_link = {}
        new_link['url'] = string.strip(url)
        new_link['title'] = string.strip(title)
        new_link['description'] = string.strip(description)
        attr_links.append(new_link)
        # Update attribute.
        self.setObjProperty('attr_links',attr_links,REQUEST['lang'])

    # ------------------------------------------------------------------------------------------
    #	ZMSLinkContainer.__del_links__: 
    #
    #	Delete links with specified IDs.
    # ------------------------------------------------------------------------------------------
    def __del_links__(self, ids, REQUEST):
      # Delete links.
      new_links = []
      old_links = self.__get_links__(REQUEST)
      count = len(old_links)
      for id in range(0,count):
        if not str(id) in ids:
          new_links.append(old_links[id].copy())
      # Update attribute.
      self.setObjProperty('attr_links',new_links,REQUEST['lang'])

    # ------------------------------------------------------------------------------------------
    #	ZMSLinkContainer.__move_link__: 
    #
    #	Moves link with specified ID in specified DIRECTION.
    # ------------------------------------------------------------------------------------------
    def __move_link__(self, id, dir, REQUEST):
      # Move link.
      links = self.__get_links__(REQUEST)
      link = links[id]
      del links[id]
      links.insert(id + dir,link)
      # Update attribute.
      self.setObjProperty('attr_links',links,REQUEST['lang'])


    """
    ###############################################################################################    
    ###  
    ###  HTML-Darstellung 
    ### 
    ###############################################################################################    
    """

    # ------------------------------------------------------------------------------------------
    #	ZMSLinkContainer.renderShort:
    #
    #	Renders short presentation of Link-Container.
    # ------------------------------------------------------------------------------------------
    def renderShort(self, REQUEST):
      s = ''
      try:
        # Retrieve properties.
        oLinkDetails = self.getLinkDetails(REQUEST)
        # Build <html>-presentation.
        if oLinkDetails:
          s = s + '\n<table cellspacing="0" cellpadding="1" border="0" width="100%" class="cmsBgColorMedium0">'
          for oLinkDetail in oLinkDetails:
            sImg, sTitle, sSpecial = self.renderShortLink(oLinkDetail,REQUEST)
            s = s + '\n<tr>'
            s = s + '\n<td width="1%" valign="top"><img src="' + sImg + '" alt="" border="0"></td>'
            s = s + '\n<td width="83%" valign="top">'
            s = s + '\n<table cellspacing="0" cellpadding="1" border="0" width="100%">'
            s = s + '\n<tr>'
            s = s + '\n<td width="1%" valign="top">' + sSpecial + '</td>'
            s = s + '\n<td width="83%" valign="top"><font style="font-size:8pt;">' + sTitle + '</font></td>'
            s = s + '\n</tr>'
            s = s + '\n</table>'
            s = s + '\n</td>'
            s = s + '\n</tr>'
          s = s + '\n</table>'
      except:
        s = ''
        s = s + '[Exception]:<br>'
        s = s + '-  Type: <strong>%s</strong><br>'%str(sys.exc_type)
        s = s + '-  Value: <strong>%s</strong><br>'%str(sys.exc_value)
        s = s + '-  Traceback: <strong>%s</strong>'%str(sys.exc_traceback)
      # Return <html>
      return s


    # ------------------------------------------------------------------------------------------
    #  ZMSLinkContainer.renderShortLink:
    #
    #  Renders short presentation of a Link.
    # ------------------------------------------------------------------------------------------
    def renderShortLink(self, oLinkDetail, REQUEST):
      
      url = oLinkDetail.getUrl()
      display_url = oLinkDetail.getDisplayUrl(self,REQUEST)
      
      special = ''
      if isInternalLink(url):
        dst = oLinkDetail.getLinkObj(self,REQUEST)
        if dst is not None:
          img = self.internal_link_gif
          display_url = '%s/manage_main?lang=%s&manage_lang=%s'%(dst.absolute_url(),REQUEST['lang'],REQUEST['manage_lang'])
          if dst.meta_type in ['ZMSFile']:
            special = dst.getMimeTypeIcon(REQUEST)
        else:
          img = self.internal_link_broken_gif
          display_url = '%s/%s/manage_links?lang=%s&manage_lang=%s'%(self.getParentNode().absolute_url(),self.id,REQUEST['lang'],REQUEST['manage_lang'])
      elif isMailLink(url):
        img = self.mail_link_gif
      else:
        img = self.external_link_gif
            
      display_title = oLinkDetail.getDisplayTitle(self,REQUEST)
      if string.lower(display_title) == 'medline':
        display_title = '<img src="%s" alt="" border="0">'%(self.MISC_ZMS + "pubmed_small.gif")
      
      if len(display_url) > 0:
        title = '<a href="%s">%s</a>'%(display_url,display_title)
      else:
        title = display_title
      
      display_description = oLinkDetail.getDisplayDescription(self,REQUEST)
      if len(display_description) > 0:
        title = title + '<br>%s'%display_description
        
      return img, title, special


    # ------------------------------------------------------------------------------------------
    #  ZMSLinkContainer.catalogText:
    #
    #  Catalog text of Link-Container (overwrite method from ZCatalogItem).
    # ------------------------------------------------------------------------------------------
    def catalogText(self, REQUEST): 
      s = ""
      if self.isVisible(REQUEST):
        linkdetails = self.getLinkDetails(REQUEST)
        for linkdetail in linkdetails:
          s = s + linkdetail.getDisplayTitle(self,REQUEST) + ' '
          s = s + linkdetail.getDisplayDescription(self,REQUEST) + ' '
      return s

###################################################################################################
